#!/usr/bin/env node

// node_modules/.pnpm/tsup@5.12.0_typescript@4.6.2/node_modules/tsup/assets/esm_shims.js
import { fileURLToPath } from "url";
import path from "path";
var getFilename = () => fileURLToPath(import.meta.url);
var getDirname = () => path.dirname(getFilename());
var __dirname = /* @__PURE__ */ getDirname();

// src/leetcode-dump.ts
import path5 from "path";
import chalk2 from "chalk";
import { program } from "commander";
import { config } from "dotenv";

// src/build.ts
import { execSync } from "child_process";
import fs from "fs";
import path2 from "path";

// src/constants.ts
var EXTS = {
  cpp: ".cpp",
  java: ".java",
  python: ".py",
  python3: ".py",
  c: ".c",
  csharp: ".cs",
  javascript: ".js",
  ruby: ".rb",
  swift: ".swift",
  golang: ".go",
  scala: ".scala",
  kotlin: ".kt",
  rust: ".rs",
  php: ".php",
  typescript: ".ts",
  racket: ".rkt",
  erlang: ".erl",
  elixir: ".ex"
};
var COMMENTS = {
  cpp: "//",
  java: "//",
  python: "#",
  python3: "#",
  c: "//",
  csharp: "//",
  javascript: "//",
  ruby: "#",
  swift: "//",
  golang: "//",
  scala: "//",
  kotlin: "//",
  rust: "//",
  php: "//",
  typescript: "//",
  racket: ";",
  erlang: "%",
  elixir: "#"
};
var LEETCODE_BASE = "https://leetcode.com";
var LEETCODE_DETAIL_BASE = LEETCODE_BASE + "/submissions/detail/";
var LEETCODE_SLUG_BASE = LEETCODE_BASE + "/problems/";
var README_TEMPLATE = `
# LeetCode Solutions

$card

## Solutions

| Problem | Difficulty | Tags | Solution |
| ------- | ---------- | ---- | -------- |
$table
`;
var DOC_TEMPLATE = `---
title: $title
---

# $title

$problem

$note

## Code

$code
`;

// src/build.ts
async function build(source, output, tmp, config2) {
  source = path2.resolve(source);
  output = path2.resolve(output);
  tmp = path2.resolve(tmp);
  console.log(`Building site to ${output}`);
  if (!fs.existsSync(source)) {
    throw new Error(`${source} does not exist`);
  }
  if (fs.existsSync(tmp)) {
    fs.rmSync(tmp, { recursive: true });
  }
  fs.mkdirSync(tmp, { recursive: true });
  fs.writeFileSync(path2.resolve(tmp, "README.md"), fix(fs.readFileSync(path2.resolve(source, "README.md"), "utf8")));
  const problems = fs.readdirSync(source).filter((name) => fs.statSync(path2.resolve(source, name)).isDirectory());
  for (const problem of problems) {
    const slug = problem.replace(/(\d+)\./, "$1").replace(/\s+/g, "-").replace("'", "").toLowerCase();
    fs.mkdirSync(path2.resolve(tmp, slug), { recursive: true });
    const doc = gen_doc(path2.resolve(source, problem));
    fs.writeFileSync(path2.resolve(tmp, slug, "index.md"), doc);
  }
  if (fs.existsSync(output)) {
    fs.rmSync(output, { recursive: true });
  }
  await build_site(tmp, output, config2);
  fs.rmSync(tmp, { recursive: true });
}
function gen_doc(src) {
  const title = path2.basename(src);
  const problem = fs.readFileSync(path2.resolve(src, "README.md"), "utf8");
  const note = fs.readFileSync(path2.resolve(src, "NOTE.md"), "utf8");
  const sources = fs.readdirSync(src).filter((name) => !name.endsWith(".md"));
  const solutions = sources.map((name) => {
    const solution = fs.readFileSync(path2.resolve(src, name), "utf8");
    return { type: path2.extname(name).slice(1), solution };
  });
  return DOC_TEMPLATE.replace(/\$title/g, title).replace(/\$note/g, note).replace(/\$problem/g, problem.replace(/#[^\n]+\n/, "<br>\n\n")).replace(/\$code/g, solutions.map(({ type, solution }) => `### ${type.toUpperCase()}

\`\`\`${type}
${solution}
\`\`\``).join("\n\n"));
}
function fix(source) {
  return source.replace(/\(\.\/\d{1,4}\.[^)]+\)/g, (match) => match.replace(/(\d+)\./, "$1").replace(/(%20)+/g, "-").replace("'", "").toLowerCase()).replace(/([^.]\/)[^.]+\.([\w]+\))/g, "$1#$2");
}
async function build_site(source, dest, config2) {
  const exe = path2.resolve(__dirname, "../node_modules/vuepress/bin/vuepress.js");
  if (config2) {
    execSync(`${exe} build ${source} --dest ${dest} --config ${config2}`, {
      stdio: "inherit"
    });
  } else {
    const conf = {
      plugins: [[path2.resolve(__dirname, "../node_modules", "@vuepress/plugin-search"), {}]],
      title: "LeetCode Solutions"
    };
    fs.mkdirSync(path2.resolve(source, ".vuepress"), { recursive: true });
    fs.writeFileSync(path2.resolve(source, ".vuepress", "config.ts"), `export default ${JSON.stringify(conf, null, 4)};`);
    execSync(`${exe} build ${source} --dest ${dest}`, {
      stdio: "inherit"
    });
  }
}

// src/dump.ts
import fs3 from "fs";
import path4 from "path";
import { Credential, LeetCode } from "leetcode-query";
import fetch from "node-fetch";
import Ora from "ora";
import TurndownService from "turndown";

// src/utils.ts
import fs2 from "fs";
import path3 from "path";
import chalk from "chalk";
function get_package_meta() {
  return JSON.parse(fs2.readFileSync(path3.join(__dirname, "../package.json"), "utf8"));
}
function readable_memory(mem) {
  const units = ["B", "KB", "MB", "GB", "TB"];
  let unit = 0;
  while (mem > 1e3) {
    mem /= 1e3;
    unit++;
  }
  return `${mem.toFixed(2)} ${units[unit]}`;
}
function sleep(ms) {
  return new Promise((resolve) => setTimeout(resolve, ms));
}
async function retry(func, max_retry2 = 3) {
  let error = null;
  for (let retry2 = 0; retry2 < max_retry2; retry2++) {
    try {
      return await func();
    } catch (err) {
      console.error(chalk.redBright(err));
      error = err;
    }
  }
  throw error;
}

// src/dump.ts
var turndown = new TurndownService().addRule("pre", {
  filter: "pre",
  replacement: (content, node) => "\n```\n" + node.textContent.replace("```", "\\`\\`\\`").trim() + "\n```\n"
}).addRule("sup", {
  filter: "sup",
  replacement: (content) => `^${content}`
});
var leetcode;
var max_retry = 3;
async function dump({
  session,
  output = path4.resolve("leetcode"),
  clean = true,
  cooldown = 250,
  timezone = "Asia/Taipei",
  pure = false,
  retry: retry2 = 3
}) {
  const { dir } = await setup({ timezone, retry: retry2, output, clean, session });
  const spinner = Ora({ text: "Scanning...", spinner: "bouncingBar" }).start();
  const { list, ac } = await get_list();
  spinner.succeed(`Scan Done. (${list.length} Problems, ${ac.length} Accepted)`);
  await sleep(cooldown);
  const table = [];
  spinner.start("Dumping Submissions...");
  for (let i = 0; i < ac.length; i++) {
    spinner.text = `Dumping Submissions... (${i + 1}/${ac.length})`;
    const { titleSlug: slug } = ac[i];
    const problem = await get_problem(slug);
    const folder = path4.resolve(dir, `${problem.questionFrontendId}. ${problem.title}`);
    if (!fs3.existsSync(folder)) {
      fs3.mkdirSync(folder, { recursive: true });
    }
    fs3.writeFileSync(path4.resolve(folder, "README.md"), `# ${problem.questionFrontendId}. ${problem.title}

Tags: ${problem.topicTags.map((t) => "`" + t.name + "`").join(", ")}

${turndown.turndown(problem.content)}`);
    fs3.writeFileSync(path4.resolve(folder, "NOTE.md"), problem.note);
    const submissions = await get_submissions(slug);
    spinner.text += ` [${Object.keys(submissions).join(", ")}]`;
    await sleep(cooldown);
    const row = [
      `[${problem.questionFrontendId}. ${problem.title}](./${encodeURI(path4.basename(folder))}) [\u{1F517}](${LEETCODE_SLUG_BASE}${slug}/)`,
      problem.difficulty,
      problem.topicTags.map((t) => "`" + t.name + "`").join(", "),
      []
    ];
    for (const lang in submissions) {
      const ext = EXTS[lang];
      const file = path4.resolve(folder, `${slug}${ext}`);
      if (!fs3.existsSync(file)) {
        const submission = await get_submission(submissions[lang].id);
        const info = `${COMMENTS[lang]} ${problem.questionFrontendId}. ${problem.title} (${new Date(+submissions[lang].timestamp * 1e3).toLocaleDateString()})
${COMMENTS[lang]} Runtime: ${submission.runtime} ms (${submission.runtime_percentile.toFixed(2)}%) Memory: ${readable_memory(submission.memory)} (${submission.memory_percentile.toFixed(2)}%) 

`;
        fs3.writeFileSync(file, `${pure ? "" : info}${submission.code}`);
        row[3].push(`[${lang}](${encodeURI(`./${path4.basename(folder)}/${slug}${ext}`)})`);
        await sleep(cooldown);
      }
    }
    row[3].sort();
    table.push(row);
  }
  spinner.succeed("Submissions Dumped.");
  fs3.writeFileSync(path4.resolve(dir, "README.md"), await create_toc(table));
}
async function setup({
  session,
  output,
  clean,
  timezone,
  retry: retry2
}) {
  process.env.TZ = timezone;
  max_retry = retry2;
  const dir = path4.resolve(output);
  if (fs3.existsSync(dir) && clean) {
    fs3.rmSync(dir, { recursive: true });
  }
  if (!fs3.existsSync(dir)) {
    fs3.mkdirSync(dir, { recursive: true });
  }
  const credential = new Credential();
  await credential.init(session);
  leetcode = new LeetCode(credential);
  return { dir };
}
async function get_list() {
  const ql = `query allQuestionsStatusesRaw { allQuestions: allQuestionsRaw { titleSlug questionFrontendId status } }`;
  const json = await retry(() => leetcode.graphql({
    query: ql,
    operationName: "allQuestionsStatusesRaw",
    variables: {}
  }), max_retry);
  const list = json.data.allQuestions;
  const ac = list.filter((item) => item.status === "ac").sort((a, b) => +a.questionFrontendId - +b.questionFrontendId);
  return { list, ac };
}
async function get_problem(slug) {
  const ql = `
    query questionData($slug: String!) {
        question(titleSlug: $slug) {
            questionFrontendId title content difficulty similarQuestions categoryTitle stats topicTags { name slug } note
        }
    }`;
  const json = (await retry(() => leetcode.graphql({ query: ql, operationName: "questionData", variables: { slug } }), max_retry)).data.question;
  json.similarQuestions = JSON.parse(json.similarQuestions);
  json.stats = JSON.parse(json.stats);
  return json;
}
async function get_submissions(slug) {
  const ql = `
    query Submissions($offset: Int!, $limit: Int!, $last: String, $slug: String!) {
        submissionList(offset: $offset, limit: $limit, lastKey: $last, questionSlug: $slug) {
            hasNext
            submissions { id statusDisplay lang runtime timestamp }
        }
    }`;
  const list = {};
  for (let i = 0; i < 10; i++) {
    const json = await retry(() => leetcode.graphql({
      query: ql,
      operationName: "Submissions",
      variables: { slug, offset: 0 * 20, limit: 20 }
    }), max_retry);
    const { submissions, hasNext } = json.data.submissionList;
    for (const submission of submissions) {
      const { id, statusDisplay, lang, runtime, timestamp } = submission;
      if (statusDisplay !== "Accepted") {
        continue;
      }
      if (!list[lang] || parseInt(runtime) < parseInt(list[lang].runtime)) {
        list[lang] = { id, timestamp, runtime };
      }
    }
    if (submissions.length === 0 || !hasNext) {
      break;
    }
  }
  return list;
}
async function get_submission(id) {
  var _a;
  const res = await retry(() => fetch(`${LEETCODE_DETAIL_BASE}${id}/`, {
    headers: {
      Cookie: `LEETCODE_SESSION=${leetcode.credential.session}`,
      Referer: LEETCODE_BASE
    }
  }), max_retry);
  const raw = await res.text();
  const data = (_a = raw.match(/var pageData = ({[^]+?});/)) == null ? void 0 : _a[1];
  const json = new Function("return " + data)();
  const result = {
    runtime: +json.runtime,
    runtime_distribution: json.runtimeDistributionFormatted ? JSON.parse(json.runtimeDistributionFormatted).distribution.map((item) => [+item[0], item[1]]) : null,
    runtime_percentile: 0,
    memory: +json.memory,
    memory_distribution: json.memoryDistributionFormatted ? JSON.parse(json.memoryDistributionFormatted).distribution.map((item) => [+item[0], item[1]]) : null,
    memory_percentile: 0,
    code: json.submissionCode
  };
  if (result.runtime_distribution) {
    result.runtime_percentile = result.runtime_distribution.reduce((acc, [usage, p]) => acc + (usage >= result.runtime ? p : 0), 0);
  }
  if (result.memory_distribution) {
    result.memory_percentile = result.memory_distribution.reduce((acc, [usage, p]) => acc + (usage >= result.memory / 1e3 ? p : 0), 0);
  }
  return result;
}
async function create_toc(table) {
  const username = await get_username();
  const card = `[![LeetCode Stats Card](https://leetcode.card.workers.dev/?username=${username})](https://leetcode.com/${username}/)`;
  const table_str = table.map(([title, difficulty, tags, solutions]) => `| ${title} | ${difficulty} | ${tags} | ${solutions.join(" \\| ")} |`).join("\n");
  return README_TEMPLATE.replace("$card", card).replace("$table", table_str);
}
async function get_username() {
  return (await retry(() => leetcode.graphql({
    query: `query globalData { userStatus { username } }`,
    operationName: "globalData",
    variables: {}
  }), max_retry)).data.userStatus.username;
}

// src/leetcode-dump.ts
config();
var package_meta = get_package_meta();
program.version(chalk2`{cyanBright ${package_meta.name}} {yellowBright v${package_meta.version}}`);
process.stdout.isTTY = true;
program.option("-s, --session <session>", "Your LeetCode Session", process.env.LEETCODE_SESSION || "process.env.LEETCODE_SESSION").option("-o, --output <path>", "Output Dir", path5.resolve("leetcode")).option("-c, --clean", "Clean Output Dir Before Start", true).option("-cd, --cooldown <ms>", "Cooldown Between Actions, in ms", "250").option("-t, --timezone <timezone>", "Your Timezone", "Asia/Taipei").option("-p, --pure", "Pure Mode, No Additional Informations to Add", false).option("-r, --retry <times>", "Times to Retry When Fail", "3").action(async () => {
  const opts = program.opts();
  await dump({
    session: opts.session,
    output: opts.output,
    clean: opts.clean,
    cooldown: +opts.cooldown,
    timezone: opts.timezone,
    pure: opts.pure,
    retry: +opts.retry
  });
});
var build_command = program.command("build").description("Build static site from dumped solutions").option("-s, --source <path>", "Source Dir", path5.resolve("leetcode")).option("-o, --output <path>", "Output Dir", path5.resolve("site")).option("-t, --temp <path>", "Temp Dir", path5.resolve("site_tmp")).option("-c, --config <path>", "Vuepress Config Path", void 0).action(() => {
  const opts = build_command.opts();
  build(opts.source, opts.output, opts.temp, opts.config);
});
program.parse();
